/*
 *	Yet Another Power Daemon
 *
 *	(c) 1996 Martin Mares, <mj@atrey.karlin.mff.cuni.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <asm/io.h>
#include <unistd.h>
#include <fcntl.h>
#include <asm/signal.h>

#define UPS_PORT 0x231

#define UPF_POWER_FAILURE 0x80
#define UPF_ONLINE 0x40

#define NAME "/etc/powerstatus"

static inline int
getups(void)
{
  return inb(UPS_PORT);
}

static int state;

static void
wrstate(void)
{
  int i;

  unlink(NAME);
  i = open(NAME, O_CREAT|O_WRONLY);
  if (i < 0)
	fprintf(stderr, "Cannot open %s!\n", NAME);
  else
	{
	  if (state)
		write(i, "FAIL\n", 5);
	  else
		write(i, "OK\n", 3);
	  close(i);
	}
}

void
main(void)
{
  int ns;

  if (getuid())
	{
	  fprintf(stderr, "powerd: Permission denied\n");
	  exit(1);
	}
  iopl(3);
  if (!(getups() & UPF_ONLINE))
	{
	  fprintf(stderr, "powerd: UPS not found! Exiting!\n");
	  exit(1);
	}
  if (getups() & UPF_POWER_FAILURE)
	{
	  printf("Waiting for the power to go up...");
	  fflush(stdout);
	  while (getups() & UPF_POWER_FAILURE)
		sleep(1);
	}
  unlink(NAME);
  ns = fork();
  if (ns < 0)
	{
	  fprintf(stderr, "powerd: fork() failed!\n");
	  exit(1);
	}
  if (ns)
	exit(0);
  for(;;)
	{
	  if (getups() & UPF_POWER_FAILURE)
		ns = 1;
	  else
		ns = 0;
	  if (ns != state)
		{
		  state = ns;
		  wrstate();
		  kill(1, SIGPWR);
		}
	  sleep(ns?5:30);
	}
}
